#include <stdint.h>
#include <math.h>
#include "datum.h"
#include "zeitgleichung.h"
#include "sonnenverlauf.h"

Sonnenverlauf_t BerechneSonnenverlauf( Datum_t Datum, double Zeitzone, double GeographischeBreite, double GeographischeLaenge ) {
	
	double Deklination;
	double Zeitgleichung;
	double Zeitdifferenz;
	double AufgangOrtszeit;
	double UntergangOrtszeit;
	double AufgangWeltzeit;
	double UntergangWeltzeit;
	const double h = -50.0 / 60.0; // Sonnenaufgang beginnt sobald der Rand der Sonne ber dem Horizont ist, nicht der Mittelpunkt!
	Sonnenverlauf_t Sonnenverlauf;
	
	Zeitgleichung = BerechneZeitgleichung( &Deklination, JahrhunderteSeit2000( Datum ) );
	
	Zeitdifferenz = 12.0 * acos( ( sin( h * M_PI / 180 ) - sin( GeographischeBreite * M_PI / 180 ) * sin( Deklination ) ) / ( cos( GeographischeBreite * M_PI / 180 ) * cos( Deklination ) ) ) / M_PI;
	
	AufgangOrtszeit = 12.0 - Zeitdifferenz - Zeitgleichung;
	AufgangWeltzeit = AufgangOrtszeit - GeographischeLaenge / 15.0;
	Sonnenverlauf.Aufgang = AufgangWeltzeit + Zeitzone;
	
	if ( Sonnenverlauf.Aufgang < 0.0 ) Sonnenverlauf.Aufgang += 24.0;
	else if ( Sonnenverlauf.Aufgang >= 24.0 ) Sonnenverlauf.Aufgang -= 24.0;
	
	UntergangOrtszeit = 12.0 + Zeitdifferenz - Zeitgleichung;
	UntergangWeltzeit = UntergangOrtszeit - GeographischeLaenge / 15.0;
	Sonnenverlauf.Untergang = UntergangWeltzeit + Zeitzone;
	
	if ( Sonnenverlauf.Untergang < 0.0 ) Sonnenverlauf.Untergang += 24.0;
	else if ( Sonnenverlauf.Untergang >= 24.0 ) Sonnenverlauf.Untergang -= 24.0;
	
	return Sonnenverlauf;
}

